@@ -445,6 +445,12 @@ bool Sql_cmd_update::update_single_table(THD *thd)
                                       (uchar *) 0);
   }
 
+  if (conds && optimizer_flag(thd, OPTIMIZER_SWITCH_SARGABLE_CASEFOLD))
+  {
+    conds= conds->top_level_transform(thd, &Item::varchar_upper_cmp_transformer,
+                                          (uchar *) 0);
+  }
+
   // Don't count on usage of 'only index' when calculating which key to use
   table->covering_keys.clear_all();
   transactional_table= table->file->has_transactions_and_rollback();
@@ -463,6 +469,13 @@ bool Sql_cmd_update::update_single_table(THD *thd)
     if (thd->binlog_for_noop_dml(transactional_table))
       DBUG_RETURN(1);
 
+    if (!thd->lex->current_select->leaf_tables_saved)
+    {
+      thd->lex->current_select->save_leaf_tables(thd);
+      thd->lex->current_select->leaf_tables_saved= true;
+      thd->lex->current_select->first_cond_optimization= 0;
+    }
+
     my_ok(thd);				// No matching records
     DBUG_RETURN(0);
   }
@@ -501,6 +514,7 @@ bool Sql_cmd_update::update_single_table(THD *thd)
     {
       thd->lex->current_select->save_leaf_tables(thd);
       thd->lex->current_select->leaf_tables_saved= true;
+      thd->lex->current_select->first_cond_optimization= 0;
     }
 
     my_ok(thd);				// No matching records
@@ -711,7 +725,6 @@ bool Sql_cmd_update::update_single_table(THD *thd)
 
       if (!(file_sort= filesort(thd, table, &fsort, fs_tracker)))
 	goto err;
-      thd->inc_examined_row_count(file_sort->examined_rows);
 
       /*
 	Filesort has already found and selected the rows we want to update,
@@ -738,7 +751,8 @@ bool Sql_cmd_update::update_single_table(THD *thd)
       explain->buf_tracker.on_scan_init();
       IO_CACHE tempfile;
       if (open_cached_file(&tempfile, mysql_tmpdir,TEMP_PREFIX,
-                           DISK_CHUNK_SIZE, MYF(MY_WME)))
+                           DISK_CHUNK_SIZE,
+                           MYF(MY_WME | MY_TRACK_WITH_LIMIT)))
         goto err;
 
       /* If quick select is used, initialize it before retrieving rows. */
@@ -779,7 +793,7 @@ bool Sql_cmd_update::update_single_table(THD *thd)
       while (likely(!(error=info.read_record())) && likely(!thd->killed))
       {
         explain->buf_tracker.on_record_read();
-        thd->inc_examined_row_count(1);
+        thd->inc_examined_row_count();
 	if (!select || (error= select->skip_record(thd)) > 0)
 	{
           if (table->file->ha_was_semi_consistent_read())
@@ -916,7 +930,7 @@ update_begin:
   while (!(error=info.read_record()) && !thd->killed)
   {
     explain->tracker.on_record_read();
-    thd->inc_examined_row_count(1);
+    thd->inc_examined_row_count();
     if (!select || select->skip_record(thd) > 0)
     {
       if (table->file->ha_was_semi_consistent_read())
@@ -1057,13 +1071,19 @@ error:
         rows_inserted++;
       }
 
+      void *save_bulk_param= thd->bulk_param;
+      thd->bulk_param= nullptr;
+
       if (table->triggers &&
           unlikely(table->triggers->process_triggers(thd, TRG_EVENT_UPDATE,
                                                      TRG_ACTION_AFTER, TRUE)))
       {
         error= 1;
+        thd->bulk_param= save_bulk_param;
+
         break;
       }
+      thd->bulk_param= save_bulk_param;
 
       if (!--limit && using_limit)
       {
@@ -1251,8 +1271,25 @@ update_end:
                   ER_THD(thd, ER_UPDATE_INFO_WITH_SYSTEM_VERSIONING),
                   (ulong) found, (ulong) updated, (ulong) rows_inserted,
                   (ulong) thd->get_stmt_da()->current_statement_warn_count());
+    thd->collect_unit_results(
+            id,
+            (thd->client_capabilities & CLIENT_FOUND_ROWS) ? found : updated);
     my_ok(thd, (thd->client_capabilities & CLIENT_FOUND_ROWS) ? found : updated,
           id, buff);
+    if (thd->get_stmt_da()->is_bulk_op())
+    {
+      /*
+        Update the diagnostics message sent to a client with number of actual
+        rows update by the statement. For bulk UPDATE operation it should be
+        done after returning from my_ok() since the final number of updated
+        rows be knows on finishing the entire bulk update statement.
+      */
+      my_snprintf(buff, sizeof(buff), ER_THD(thd, ER_UPDATE_INFO),
+                  (ulong) thd->get_stmt_da()->affected_rows(),
+                  (ulong) thd->get_stmt_da()->affected_rows(),
+                  (ulong) thd->get_stmt_da()->current_statement_warn_count());
+      thd->get_stmt_da()->set_message(buff);
+    }
     DBUG_PRINT("info",("%ld records updated", (long) updated));
   }
   thd->count_cuted_fields= CHECK_FIELD_IGNORE;		/* calc cuted fields */
@@ -1261,6 +1298,7 @@ update_end:
   {
     thd->lex->current_select->save_leaf_tables(thd);
     thd->lex->current_select->leaf_tables_saved= true;
+    thd->lex->current_select->first_cond_optimization= 0;
   }
   ((multi_update *)result)->set_found(found);
   ((multi_update *)result)->set_updated(updated);
@@ -1583,8 +1621,8 @@ bool Multiupdate_prelocking_strategy::handle_end(THD *thd)
     DBUG_RETURN(true);
 
   List<Item> *fields= &lex->first_select_lex()->item_list;
-  if (setup_fields_with_no_wrap(thd, Ref_ptr_array(),
-                                *fields, MARK_COLUMNS_WRITE, 0, 0))
+  if (setup_fields_with_no_wrap(thd, Ref_ptr_array(), *fields,
+                                MARK_COLUMNS_WRITE, 0, 0, THD_WHERE::SET_LIST))
     DBUG_RETURN(1);
 
   // Check if we have a view in the list ...
@@ -3101,5 +3139,6 @@ bool Sql_cmd_update::execute_inner(THD *thd)
     delete result;
   }
 
+  status_var_add(thd->status_var.rows_sent, thd->get_sent_row_count());
   return res;
 }
