@@ -41,12 +41,14 @@ static ulong opt_myisam_block_size;
 /* bits in myisam_recover_options */
 const char *myisam_recover_names[] =
 { "DEFAULT", "BACKUP", "FORCE", "QUICK", "BACKUP_ALL", "OFF", NullS};
-TYPELIB myisam_recover_typelib= CREATE_TYPELIB_FOR(myisam_recover_names);
+TYPELIB myisam_recover_typelib= {array_elements(myisam_recover_names)-1,"",
+				 myisam_recover_names, NULL};
 
 const char *myisam_stats_method_names[] = {"NULLS_UNEQUAL", "NULLS_EQUAL",
                                            "NULLS_IGNORED", NullS};
-TYPELIB myisam_stats_method_typelib=
-                CREATE_TYPELIB_FOR(myisam_stats_method_names);
+TYPELIB myisam_stats_method_typelib= {
+  array_elements(myisam_stats_method_names) - 1, "",
+  myisam_stats_method_names, NULL};
 
 static MYSQL_SYSVAR_ULONG(block_size, opt_myisam_block_size,
   PLUGIN_VAR_READONLY | PLUGIN_VAR_RQCMDARG,
@@ -85,7 +87,7 @@ static MYSQL_SYSVAR_BOOL(use_mmap, opt_myisam_use_mmap, PLUGIN_VAR_NOCMDARG,
 
 static MYSQL_SYSVAR_ULONGLONG(mmap_size, myisam_mmap_size,
   PLUGIN_VAR_RQCMDARG|PLUGIN_VAR_READONLY, "Restricts the total memory "
-  "used for memory mapping of MySQL tables", NULL, NULL,
+  "used for memory mapping of MyISAM tables", NULL, NULL,
   SIZE_T_MAX, MEMMAP_EXTRA_MARGIN, SIZE_T_MAX, 1);
 
 static MYSQL_THDVAR_ENUM(stats_method, PLUGIN_VAR_RQCMDARG,
@@ -365,13 +367,13 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
   while (recpos < (uint) share->stored_rec_length)
   {
     Field **field, *found= 0;
-    minpos= share->stored_rec_length;
+    minpos= share->reclength;
     length= 0;
 
     for (field= table_arg->field; *field; field++)
     {
       if ((fieldpos= (*field)->offset(record)) >= recpos &&
-          fieldpos < minpos)
+          fieldpos <= minpos)
       {
         /* skip null fields */
         if (!(temp_length= (*field)->pack_length_in_rec()))
@@ -708,16 +710,6 @@ my_bool mi_killed_in_mariadb(MI_INFO *info)
   return (((TABLE*) (info->external_ref))->in_use->killed != 0);
 }
 
-static void init_compute_vcols(void *table)
-{
-  /*
-    To evaluate vcols we must have current_thd set.
-    This will set current_thd in all threads to the same THD, but it's
-    safe, because vcols are always evaluated under info->s->intern_lock.
-  */
-  set_current_thd(static_cast<TABLE *>(table)->in_use);
-}
-
 static int compute_vcols(MI_INFO *info, uchar *record, int keynum)
 {
   int error= 0;
@@ -1036,7 +1028,6 @@ int ha_myisam::setup_vcols_for_repair(HA_CHECK *param)
   }
   DBUG_ASSERT(file->s->base.reclength < file->s->vreclength ||
               !table->s->stored_fields);
-  param->init_fix_record= init_compute_vcols;
   param->fix_record= compute_vcols;
   table->use_all_columns();
   return 0;
@@ -1280,6 +1271,25 @@ int ha_myisam::optimize(THD* thd, HA_CHECK_OPT *check_opt)
 }
 
 
+/*
+  Set current_thd() for parallel worker thread
+  This is needed to evaluate vcols as we must have current_thd set.
+  This will set current_thd in all threads to the same THD, but it's
+  safe, because vcols are always evaluated under info->s->intern_lock.
+
+  This is also used temp_file_size_cb_func() to tmp_space_usage by THD.
+*/
+
+C_MODE_START
+void myisam_setup_thd_for_repair_thread(void *arg)
+{
+  THD *thd= (THD*) arg;
+  DBUG_ASSERT(thd->shared_thd);
+  set_current_thd(thd);
+}
+C_MODE_END
+
+
 int ha_myisam::repair(THD *thd, HA_CHECK &param, bool do_optimize)
 {
   int error=0;
@@ -1351,8 +1361,18 @@ int ha_myisam::repair(THD *thd, HA_CHECK &param, bool do_optimize)
       {
         /* TODO: respect myisam_repair_threads variable */
         thd_proc_info(thd, "Parallel repair");
-        error = mi_repair_parallel(&param, file, fixed_name,
-                                   MY_TEST(param.testflag & T_QUICK));
+        param.testflag|= T_REP_PARALLEL;
+        /*
+          Ensure that all threads are using the same THD. This is needed
+          to get limit of tmp files to work
+        */
+        param.init_repair_thread= myisam_setup_thd_for_repair_thread;
+        param.init_repair_thread_arg= table->in_use;
+        /* Mark that multiple threads are using the thd */
+        table->in_use->shared_thd= 1;
+        error= mi_repair_parallel(&param, file, fixed_name,
+                                  MY_TEST(param.testflag & T_QUICK));
+        table->in_use->shared_thd= 0;
       }
       else
       {
@@ -1498,7 +1518,7 @@ int ha_myisam::assign_to_keycache(THD* thd, HA_CHECK_OPT *check_opt)
     param->db_name=    table->s->db.str;
     param->table_name= table->s->table_name.str;
     param->testflag= 0;
-    mi_check_print_error(param, "%s", errmsg);
+    mi_check_print_error(param, errmsg);
   }
   DBUG_RETURN(error);
 }
@@ -1564,7 +1584,7 @@ int ha_myisam::preload_keys(THD* thd, HA_CHECK_OPT *check_opt)
     param->db_name=    table->s->db.str;
     param->table_name= table->s->table_name.str;
     param->testflag=   0;
-    mi_check_print_error(param, "%s", errmsg);
+    mi_check_print_error(param, errmsg);
     DBUG_RETURN(error);
   }
 }
